@@ -301,7 +301,10 @@ md5_final( MD5_CONTEXT *hd )
     #define X(a) do { *p++ = hd->##a	  ; *p++ = hd->##a >> 8;      \
 		      *p++ = hd->##a >> 16; *p++ = hd->##a >> 24; } while(0)
   #else /* little endian */
-    #define X(a) do { *(u32*)p = hd->##a ; p += 4; } while(0)
+    /*#define X(a) do { *(u32*)p = hd->##a ; p += 4; } while(0)*/
+    /* Unixware's cpp doesn't like the above construct so we do it his way:
+     * (reported by Allan Clark) */
+    #define X(a) do { *(u32*)p = (*hd).a ; p += 4; } while(0)
   #endif
     X(A);
     X(B);
@@ -355,7 +358,7 @@ md5_get_info( int algo, size_t *contextsize,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "MD5 ($Revision: 1.13 $)";
+const char * const gnupgext_version = "MD5 ($Revision: 1.14 $)";
 
 static struct {
     int class;
