@@ -33,8 +33,6 @@
  * a key consisting of words and dots.	Because the lookup is only
  * done in an interactive mode on a user request (when she enters a "?"
  * as response to a prompt) we can use a simple search through the list.
- * Translators should use the key as msgid, this is to keep the msgid short
- * and to allow for easy changing of the helptexts.
  *
  * Mini glossary:
  *
@@ -45,182 +43,186 @@ static struct helptexts { const char *key; const char *help; } helptexts[] = {
 
 /* begin of list */
 
-{ N_("edit_ownertrust.value"),
+{ "edit_ownertrust.value", N_(
 "It's up to you to assign a value here; this value will never be exported\n"
 "to any 3rd party.  We need it to implement the web-of-trust; it has nothing\n"
 "to do with the (implicitly created) web-of-certificates."
-},
+)},
 
-{ N_("revoked_key.override"),
+{ "revoked_key.override", N_(
 "If you want to use this revoked key anyway, answer \"yes\"."
-},
+)},
 
-{ N_("untrusted_key.override"),
+{ "untrusted_key.override", N_(
 "If you want to use this untrusted key anyway, answer \"yes\"."
-},
+)},
 
-{ N_("pklist.user_id.enter"),
-"Enter the user id of the addressee to whom you want to send the message."
-},
+{ "pklist.user_id.enter", N_(
+"Enter the user ID of the addressee to whom you want to send the message."
+)},
 
-{ N_("keygen.algo"),
+{ "keygen.algo", N_(
 "Select the algorithm to use.\n"
+"\n"
 "DSA (aka DSS) is the digital signature algorithm which can only be used\n"
 "for signatures.  This is the suggested algorithm because verification of\n"
 "DSA signatures are much faster than those of ElGamal.\n"
-"ElGamal is a algorithm which can be used for signatures and encryption.\n"
-"OpenPGP distunguishs between two flavors of this algorithms: a encrypt only\n"
+"\n"
+"ElGamal is an algorithm which can be used for signatures and encryption.\n"
+"OpenPGP distinguishs between two flavors of this algorithms: an encrypt only\n"
 "and a sign+encrypt; actually it is the same, but some parameters must be\n"
 "selected in a special way to create a safe key for signatures: this program\n"
-"does this but other OpenPGP implemenations are not required to understand\n"
+"does this but other OpenPGP implementations are not required to understand\n"
 "the signature+encryption flavor.\n"
+"\n"
 "The first (primary) key must always be a key which is capable of signing;\n"
-"this is the reason why the encryption only ElGamal key is disabled in this."
-},
+"this is the reason why the encryption only ElGamal key is not available in\n"
+"this menu."
+)},
 
 
-{ N_("keygen.algo.elg_se"),
+{ "keygen.algo.elg_se", N_(
 "Although these keys are defined in RFC2440 they are not suggested\n"
 "because they are not supported by all programs and signatures created\n"
 "with them are quite large and very slow to verify."
-},
+)},
 
 
-{ N_("keygen.size"),
+{ "keygen.size", N_(
  "Enter the size of the key"
-},
+)},
 
-{ N_("keygen.size.huge.okay"),
+{ "keygen.size.huge.okay", N_(
  "Answer \"yes\" or \"no\""
-},
+)},
 
 
-{ N_("keygen.size.large.okay"),
+{ "keygen.size.large.okay", N_(
  "Answer \"yes\" or \"no\""
-},
+)},
 
 
-{ N_("keygen.valid"),
- "Enter the required value as shown in the pronpt.\n"
+{ "keygen.valid", N_(
+ "Enter the required value as shown in the prompt.\n"
  "It is possible to enter a ISO date (YYYY-MM-DD) but you won't\n"
  "get a good error response - instead the system tries to interpret\n"
  "the given value as an interval."
-},
+)},
 
-{ N_("keygen.valid.okay"),
+{ "keygen.valid.okay", N_(
  "Answer \"yes\" or \"no\""
-},
+)},
 
 
-{ N_("keygen.name"),
+{ "keygen.name", N_(
  "Enter the name of the key holder"
-},
+)},
 
 
-{ N_("keygen.email"),
+{ "keygen.email", N_(
  "please enter an optional but highly suggested email address"
-},
+)},
 
-{ N_("keygen.comment"),
+{ "keygen.comment", N_(
  "Please enter an optional comment"
-},
+)},
 
 
-{ N_("keygen.userid.cmd"),
+{ "keygen.userid.cmd", N_(
  ""
 "N  to change the name.\n"
 "C  to change the comment.\n"
 "E  to change the email address.\n"
 "O  to continue with key generation.\n"
 "Q  to to quit the key generation."
-},
+)},
 
-{ N_("keygen.sub.okay"),
+{ "keygen.sub.okay", N_(
  "Answer \"yes\" (or just \"y\") if it is okay to generate the sub key."
-},
+)},
 
-{ N_("sign_uid.okay"),
+{ "sign_uid.okay", N_(
  "Answer \"yes\" or \"no\""
-},
+)},
 
 
-{ N_("change_passwd.empty.okay"),
+{ "change_passwd.empty.okay", N_(
  "Answer \"yes\" or \"no\""
-},
+)},
 
 
-{ N_("keyedit.save.okay"),
+{ "keyedit.save.okay", N_(
  "Answer \"yes\" or \"no\""
-},
+)},
 
 
-{ N_("keyedit.cancel.okay"),
+{ "keyedit.cancel.okay", N_(
  "Answer \"yes\" or \"no\""
-},
+)},
 
-{ N_("keyedit.sign_all.okay"),
+{ "keyedit.sign_all.okay", N_(
  "Answer \"yes\" is you want to sign ALL the user IDs"
-},
+)},
 
-{ N_("keyedit.remove.uid.okay"),
+{ "keyedit.remove.uid.okay", N_(
  "Answer \"yes\" if you really want to delete this user ID.\n"
  "All certificates are then also lost!"
-},
+)},
 
-{ N_("keyedit.remove.subkey.okay"),
+{ "keyedit.remove.subkey.okay", N_(
  "Answer \"yes\" if it is okay to delete the subkey"
-},
+)},
 
 
-{ N_("keyedit.delsig.valid"),
+{ "keyedit.delsig.valid", N_(
  "This is a valid signature on the key; you normally don't want\n"
- "to delete this signature may be important to establish a trust\n"
- "connection to the key or another key certified by this key."
-},
-{ N_("keyedit.delsig.unknown"),
+ "to delete this signature because it may be important to establish a\n"
+ "trust connection to the key or another key certified by this key."
+)},
+{ "keyedit.delsig.unknown", N_(
  "This signature can't be checked because you don't have the\n"
  "corresponding key.  You should postpone its deletion until you\n"
  "know which key was used because this signing key might establish"
  "a trust connection through another already certified key."
-},
-{ N_("keyedit.delsig.invalid"),
+)},
+{ "keyedit.delsig.invalid", N_(
  "The signature is not valid.  It does make sense to remove it from\n"
  "your keyring."
-},
-{ N_("keyedit.delsig.selfsig"),
+)},
+{ "keyedit.delsig.selfsig", N_(
  "This is a signature which binds the user ID to the key. It is\n"
  "usually not a good idea to remove such a signature.  Actually\n"
  "GnuPG might not be able to use this key anymore.  So do this\n"
  "only if this self-signature is for some reason not valid and\n"
  "a second one is available."
-},
+)},
 
 
-{ N_("passphrase.enter"),
+{ "passphrase.enter", N_(
  ""
 "Please enter the passhrase; this is a secret sentence \n"
 "  Blurb, blurb,.... "
-},
+)},
 
 
-{ N_("passphrase.repeat"),
+{ "passphrase.repeat", N_(
  "Please repeat the last passphrase, so you are sure what you typed in."
-},
+)},
 
-{ N_("detached_signature.filename"),
+{ "detached_signature.filename", N_(
  "Give the name fo the file to which the signature applies"
-},
+)},
 
 /* openfile.c (overwrite_filep) */
-{ N_("openfile.overwrite.okay"),
+{ "openfile.overwrite.okay", N_(
  "Answer \"yes\" if it is okay to overwrite the file"
-},
+)},
 
 /* openfile.c (ask_outfile_name) */
-{ N_("openfile.askoutname"),
+{ "openfile.askoutname", N_(
  "Please enter a new filename. If you just hit RETURN the default\n"
  "file (which is shown in brackets) will be used."
-},
+)},
 
 /* end of list */
 { NULL, NULL } };
@@ -234,20 +236,15 @@ display_online_help( const char *keyword )
     if( !keyword )
 	tty_printf(_("No help available") );
     else {
-	const char *p = _(keyword);
+	const char *p;
+	int i;
 
-	if( strcmp( p, keyword ) )
-	    tty_printf("%s", p );
-	else {
-	    int i;
-
-	    for(i=0; (p=helptexts[i].key) && strcmp( p, keyword ); i++ )
-		;
-	    if( !p || !*helptexts[i].help )
-		tty_printf(_("No help available for `%s'"), keyword );
-	    else
-		tty_printf("%s", helptexts[i].help );
-	}
+	for(i=0; (p=helptexts[i].key) && strcmp( p, keyword ); i++ )
+	    ;
+	if( !p || !*helptexts[i].help )
+	    tty_printf(_("No help available for `%s'"), keyword );
+	else
+	    tty_printf("%s", _(helptexts[i].help) );
     }
     tty_printf("\n");
 }
