@@ -359,7 +359,7 @@ import_one( const char *fname, KBNODE keyblock, int fast )
 	putc('\n', stderr);
     }
     if( !uidnode ) {
-	log_error( _("key %08lX: no user id\n"), (ulong)keyid[1]);
+	log_error( _("key %08lX: no user ID\n"), (ulong)keyid[1]);
 	return 0;
     }
 
@@ -373,7 +373,7 @@ import_one( const char *fname, KBNODE keyblock, int fast )
 
     if( !delete_inv_parts( fname, keyblock, keyid ) ) {
 	if( !opt.quiet ) {
-	    log_info( _("key %08lX: no valid user ids\n"),
+	    log_info( _("key %08lX: no valid user IDs\n"),
 							(ulong)keyid[1]);
 	    log_info(_("this may be caused by a missing self-signature\n"));
 	}
@@ -466,10 +466,10 @@ import_one( const char *fname, KBNODE keyblock, int fast )
 	    /* we are ready */
 	    if( !opt.quiet ) {
 		if( n_uids == 1 )
-		    log_info( _("key %08lX: 1 new user-id\n"),
+		    log_info( _("key %08lX: 1 new user ID\n"),
 					     (ulong)keyid[1]);
 		else if( n_uids )
-		    log_info( _("key %08lX: %d new user-ids\n"),
+		    log_info( _("key %08lX: %d new user IDs\n"),
 					     (ulong)keyid[1], n_uids );
 		if( n_sigs == 1 )
 		    log_info( _("key %08lX: 1 new signature\n"),
@@ -551,7 +551,7 @@ import_secret_one( const char *fname, KBNODE keyblock )
     }
     stats.secret_read++;
     if( !uidnode ) {
-	log_error( _("key %08lX: no user id\n"), (ulong)keyid[1]);
+	log_error( _("key %08lX: no user ID\n"), (ulong)keyid[1]);
 	return 0;
     }
 
@@ -708,13 +708,13 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
 	    if( (sig->sig_class&~3) == 0x10 ) {
 		KBNODE unode = find_prev_kbnode( keyblock, n, PKT_USER_ID );
 		if( !unode )  {
-		    log_error( _("key %08lX: no user-id for signature\n"),
+		    log_error( _("key %08lX: no user ID for signature\n"),
 					    (ulong)keyid[1]);
 		    return -1;	/* the complete keyblock is invalid */
 		}
 		rc = check_key_signature( keyblock, n, NULL);
 		if( rc ) {
-		    log_error( rc == G10ERR_PUBKEY_ALGO ?
+		    log_info( rc == G10ERR_PUBKEY_ALGO ?
 			 _("key %08lX: unsupported public key algorithm\n"):
 			 _("key %08lX: invalid self-signature\n"),
 				     (ulong)keyid[1]);
@@ -731,14 +731,14 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
 						 n, PKT_SECRET_SUBKEY );
 
 		if( !knode ) {
-		    log_error( _("key %08lX: no subkey for key binding\n"),
+		    log_info( _("key %08lX: no subkey for key binding\n"),
 					    (ulong)keyid[1]);
 		    n->flag |= 4; /* delete this */
 		}
 		else {
 		    rc = check_key_signature( keyblock, n, NULL);
 		    if( rc ) {
-			log_error(  rc == G10ERR_PUBKEY_ALGO ?
+			log_info(  rc == G10ERR_PUBKEY_ALGO ?
 			   _("key %08lX: unsupported public key algorithm\n"):
 			   _("key %08lX: invalid subkey binding\n"),
 					 (ulong)keyid[1]);
@@ -766,7 +766,7 @@ mark_non_selfsigned_uids_valid( KBNODE keyblock, u32 *kid )
 	if( node->pkt->pkttype == PKT_USER_ID && !(node->flag & 1) ) {
 	    if( node->next && node->next->pkt->pkttype == PKT_SIGNATURE ) {
 		node->flag |= 1;
-		log_info( _("key %08lX: accepted non self-signed userid '"),
+		log_info( _("key %08lX: accepted non self-signed user ID '"),
 							 (ulong)kid[1]);
 		print_string( log_stream(), node->pkt->pkt.user_id->name,
 					    node->pkt->pkt.user_id->len, 0 );
@@ -795,7 +795,7 @@ delete_inv_parts( const char *fname, KBNODE keyblock, u32 *keyid )
 	    uid_seen = 1;
 	    if( (node->flag & 2) || !(node->flag & 1) ) {
 		if( opt.verbose ) {
-		    log_info( _("key %08lX: skipped userid '"),
+		    log_info( _("key %08lX: skipped user ID '"),
 							 (ulong)keyid[1]);
 		    print_string( stderr, node->pkt->pkt.user_id->name,
 				      node->pkt->pkt.user_id->len, 0 );
@@ -1052,30 +1052,6 @@ merge_blocks( const char *fname, KBNODE keyblock_orig, KBNODE keyblock,
 	}
     }
 
-    /* merge subkey certificates */
-    for(onode=keyblock_orig->next; onode; onode=onode->next ) {
-	if( !(onode->flag & 1)
-	    &&	(   onode->pkt->pkttype == PKT_PUBLIC_SUBKEY
-		 || onode->pkt->pkttype == PKT_SECRET_SUBKEY) ) {
-	    /* find the subkey in the imported keyblock */
-	    for(node=keyblock->next; node; node=node->next ) {
-		if( node->pkt->pkttype == PKT_PUBLIC_SUBKEY
-		    && !cmp_public_keys( onode->pkt->pkt.public_key,
-					  node->pkt->pkt.public_key ) )
-		    break;
-		else if( node->pkt->pkttype == PKT_SECRET_SUBKEY
-		    && !cmp_secret_keys( onode->pkt->pkt.secret_key,
-					  node->pkt->pkt.secret_key ) )
-		    break;
-	    }
-	    if( node ) { /* found: merge */
-		rc = merge_keysigs( onode, node, n_sigs, fname, keyid );
-		if( rc )
-		    return rc;
-	    }
-	}
-    }
-
     /*	add new subkeys */
     for(node=keyblock->next; node; node=node->next ) {
 	onode = NULL;
@@ -1109,6 +1085,31 @@ merge_blocks( const char *fname, KBNODE keyblock_orig, KBNODE keyblock,
 	}
     }
 
+    /* merge subkey certificates */
+    for(onode=keyblock_orig->next; onode; onode=onode->next ) {
+	if( !(onode->flag & 1)
+	    &&	(   onode->pkt->pkttype == PKT_PUBLIC_SUBKEY
+		 || onode->pkt->pkttype == PKT_SECRET_SUBKEY) ) {
+	    /* find the subkey in the imported keyblock */
+	    for(node=keyblock->next; node; node=node->next ) {
+		if( node->pkt->pkttype == PKT_PUBLIC_SUBKEY
+		    && !cmp_public_keys( onode->pkt->pkt.public_key,
+					  node->pkt->pkt.public_key ) )
+		    break;
+		else if( node->pkt->pkttype == PKT_SECRET_SUBKEY
+		    && !cmp_secret_keys( onode->pkt->pkt.secret_key,
+					  node->pkt->pkt.secret_key ) )
+		    break;
+	    }
+	    if( node ) { /* found: merge */
+		rc = merge_keysigs( onode, node, n_sigs, fname, keyid );
+		if( rc )
+		    return rc;
+	    }
+	}
+    }
+
+
     return 0;
 }
 
@@ -1176,7 +1177,7 @@ merge_sigs( KBNODE dst, KBNODE src, int *n_sigs,
 
     assert(dst->pkt->pkttype == PKT_USER_ID );
     assert(src->pkt->pkttype == PKT_USER_ID );
-    /* at least a self signature comes next to the user-ids */
+    /* at least a self signature comes next to the user IDs */
     assert(src->next->pkt->pkttype != PKT_USER_ID );
     if( dst->next->pkt->pkttype == PKT_USER_ID ) {
 	log_error( _("key %08lX: our copy has no self-signature\n"),
@@ -1188,6 +1189,9 @@ merge_sigs( KBNODE dst, KBNODE src, int *n_sigs,
     for(n=src->next; n && n->pkt->pkttype != PKT_USER_ID; n = n->next ) {
 	if( n->pkt->pkttype != PKT_SIGNATURE )
 	    continue;
+	if( n->pkt->pkt.signature->sig_class == 0x18
+	    || n->pkt->pkt.signature->sig_class == 0x28 )
+	    continue; /* skip signatures which are only valid on subkeys */
 	found = 0;
 	for(n2=dst->next; n2 && n2->pkt->pkttype != PKT_USER_ID; n2 = n2->next){
 	    if( n2->pkt->pkttype == PKT_SIGNATURE
