@@ -157,7 +157,7 @@ print_and_check_one_sig( KBNODE keyblock, KBNODE node,
 	else {
 	    size_t n;
 	    char *p = get_user_id( sig->keyid, &n );
-	    tty_print_string( p, n > 40? 40 : n );
+	    tty_print_utf8_string( p, n > 40? 40 : n );
 	    m_free(p);
 	}
 	tty_printf("\n");
@@ -192,7 +192,7 @@ check_all_keysigs( KBNODE keyblock, int only_selected )
 		selected = (node->flag & NODFLG_SELUID);
 	    if( selected ) {
 		tty_printf("uid  ");
-		tty_print_string( uid->name, uid->len );
+		tty_print_utf8_string( uid->name, uid->len );
 		tty_printf("\n");
 		if( anyuid && !has_selfsig )
 		    mis_selfsig++;
@@ -228,9 +228,9 @@ check_all_keysigs( KBNODE keyblock, int only_selected )
     else if( oth_err )
 	tty_printf(_("%d signatures not checked due to errors\n"), oth_err );
     if( mis_selfsig == 1 )
-	tty_printf(_("1 user id without valid self-signature detected\n"));
+	tty_printf(_("1 user ID without valid self-signature detected\n"));
     else if( mis_selfsig  )
-	tty_printf(_("%d user ids without valid self-signatures detected\n"),
+	tty_printf(_("%d user IDs without valid self-signatures detected\n"),
 								    mis_selfsig);
 
     return inv_sigs || no_key || oth_err || mis_selfsig;
@@ -330,7 +330,7 @@ sign_uids( KBNODE keyblock, STRLIST locusr, int *ret_modified, int local )
 	     "Are you really sure that you want to sign this key\n"
 	     "with your key: \""));
 	p = get_user_id( sk_keyid, &n );
-	tty_print_string( p, n );
+	tty_print_utf8_string( p, n );
 	m_free(p); p = NULL;
 	tty_printf("\"\n\n");
 
@@ -538,13 +538,15 @@ fix_keyblock( KBNODE keyblock )
 }
 
 /****************
- * Menu driven key editor
+ * Menu driven key editor.  If sign_mode is true semi-automatical signing
+ * will be performed. commands are ignore in this case
  *
  * Note: to keep track of some selection we use node->mark MARKBIT_xxxx.
  */
 
 void
-keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
+keyedit_menu( const char *username, STRLIST locusr, STRLIST commands,
+						    int sign_mode )
 {
     enum cmdids { cmdNONE = 0,
 	   cmdQUIT, cmdHELP, cmdFPR, cmdLIST, cmdSELUID, cmdCHECK, cmdSIGN,
@@ -552,44 +554,45 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 	   cmdDEBUG, cmdSAVE, cmdADDUID, cmdDELUID, cmdADDKEY, cmdDELKEY,
 	   cmdTOGGLE, cmdSELKEY, cmdPASSWD, cmdTRUST, cmdPREF, cmdEXPIRE,
 	   cmdENABLEKEY, cmdDISABLEKEY,
-	   cmdNOP };
+	   cmdINVCMD, cmdNOP };
     static struct { const char *name;
 		    enum cmdids id;
 		    int need_sk;
+		    int signmode;
 		    const char *desc;
 		  } cmds[] = {
-	{ N_("quit")    , cmdQUIT   , 0, N_("quit this menu") },
-	{ N_("q")       , cmdQUIT   , 0, NULL   },
-	{ N_("save")    , cmdSAVE   , 0, N_("save and quit") },
-	{ N_("help")    , cmdHELP   , 0, N_("show this help") },
-	{    "?"        , cmdHELP   , 0, NULL   },
-	{ N_("fpr")     , cmdFPR    , 0, N_("show fingerprint") },
-	{ N_("list")    , cmdLIST   , 0, N_("list key and user ids") },
-	{ N_("l")       , cmdLIST   , 0, NULL   },
-	{ N_("uid")     , cmdSELUID , 0, N_("select user id N") },
-	{ N_("key")     , cmdSELKEY , 0, N_("select secondary key N") },
-	{ N_("check")   , cmdCHECK  , 0, N_("list signatures") },
-	{ N_("c")       , cmdCHECK  , 0, NULL },
-	{ N_("sign")    , cmdSIGN   , 0, N_("sign the key") },
-	{ N_("s")       , cmdSIGN   , 0, NULL },
-	{ N_("lsign")   , cmdLSIGN  , 0, N_("sign the key locally") },
-	{ N_("debug")   , cmdDEBUG  , 0, NULL },
-	{ N_("adduid")  , cmdADDUID , 1, N_("add a user id") },
-	{ N_("deluid")  , cmdDELUID , 0, N_("delete user id") },
-	{ N_("addkey")  , cmdADDKEY , 1, N_("add a secondary key") },
-	{ N_("delkey")  , cmdDELKEY , 0, N_("delete a secondary key") },
-	{ N_("delsig")  , cmdDELSIG , 0, N_("delete signatures") },
-	{ N_("expire")  , cmdEXPIRE , 1, N_("change the expire date") },
-	{ N_("toggle")  , cmdTOGGLE , 1, N_("toggle between secret "
-					    "and public key listing") },
-	{ N_("t"     )  , cmdTOGGLE , 1, NULL },
-	{ N_("pref")    , cmdPREF  , 0, N_("list preferences") },
-	{ N_("passwd")  , cmdPASSWD , 1, N_("change the passphrase") },
-	{ N_("trust")   , cmdTRUST , 0, N_("change the ownertrust") },
-	{ N_("revsig")  , cmdREVSIG , 0, N_("revoke signatures") },
-	{ N_("revkey")  , cmdREVKEY , 1, N_("revoke a secondary key") },
-	{ N_("disable") , cmdDISABLEKEY , 0, N_("disable a key") },
-	{ N_("enable")  , cmdENABLEKEY , 0, N_("enable a key") },
+	{ N_("quit")    , cmdQUIT      , 0,1, N_("quit this menu") },
+	{ N_("q")       , cmdQUIT      , 0,1, NULL   },
+	{ N_("save")    , cmdSAVE      , 0,1, N_("save and quit") },
+	{ N_("help")    , cmdHELP      , 0,1, N_("show this help") },
+	{    "?"        , cmdHELP      , 0,1, NULL   },
+	{ N_("fpr")     , cmdFPR       , 0,1, N_("show fingerprint") },
+	{ N_("list")    , cmdLIST      , 0,1, N_("list key and user IDs") },
+	{ N_("l")       , cmdLIST      , 0,1, NULL   },
+	{ N_("uid")     , cmdSELUID    , 0,1, N_("select user ID N") },
+	{ N_("key")     , cmdSELKEY    , 0,0, N_("select secondary key N") },
+	{ N_("check")   , cmdCHECK     , 0,1, N_("list signatures") },
+	{ N_("c")       , cmdCHECK     , 0,1, NULL },
+	{ N_("sign")    , cmdSIGN      , 0,1, N_("sign the key") },
+	{ N_("s")       , cmdSIGN      , 0,1, NULL },
+	{ N_("lsign")   , cmdLSIGN     , 0,1, N_("sign the key locally") },
+	{ N_("debug")   , cmdDEBUG     , 0,0, NULL },
+	{ N_("adduid")  , cmdADDUID    , 1,0, N_("add a user ID") },
+	{ N_("deluid")  , cmdDELUID    , 0,0, N_("delete user ID") },
+	{ N_("addkey")  , cmdADDKEY    , 1,0, N_("add a secondary key") },
+	{ N_("delkey")  , cmdDELKEY    , 0,0, N_("delete a secondary key") },
+	{ N_("delsig")  , cmdDELSIG    , 0,0, N_("delete signatures") },
+	{ N_("expire")  , cmdEXPIRE    , 1,0, N_("change the expire date") },
+	{ N_("toggle")  , cmdTOGGLE    , 1,0, N_("toggle between secret "
+						 "and public key listing") },
+	{ N_("t"     )  , cmdTOGGLE    , 1,0, NULL },
+	{ N_("pref")    , cmdPREF      , 0,0,  N_("list preferences") },
+	{ N_("passwd")  , cmdPASSWD    , 1,0, N_("change the passphrase") },
+	{ N_("trust")   , cmdTRUST     , 0,0,  N_("change the ownertrust") },
+	{ N_("revsig")  , cmdREVSIG    , 0,0, N_("revoke signatures") },
+	{ N_("revkey")  , cmdREVKEY    , 1,0, N_("revoke a secondary key") },
+	{ N_("disable") , cmdDISABLEKEY, 0,0, N_("disable a key") },
+	{ N_("enable")  , cmdENABLEKEY , 0,0, N_("enable a key") },
 
     { NULL, cmdNONE } };
     enum cmdids cmd = 0;
@@ -612,18 +615,27 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 	goto leave;
     }
 
-    /* first try to locate it as secret key */
-    rc = find_secret_keyblock_byname( &sec_keyblockpos, username );
-    if( !rc ) {
-	rc = read_keyblock( &sec_keyblockpos, &sec_keyblock );
-	if( rc ) {
-	    log_error("%s: secret keyblock read problem: %s\n",
-					    username, g10_errstr(rc));
-	    goto leave;
+    if( sign_mode ) {
+	commands = NULL;
+	append_to_strlist( &commands, sign_mode == 1? "sign":"lsign" );
+	have_commands = 1;
+    }
+
+
+    if( !sign_mode ) {
+	/* first try to locate it as secret key */
+	rc = find_secret_keyblock_byname( &sec_keyblockpos, username );
+	if( !rc ) {
+	    rc = read_keyblock( &sec_keyblockpos, &sec_keyblock );
+	    if( rc ) {
+		log_error("%s: secret keyblock read problem: %s\n",
+						username, g10_errstr(rc));
+		goto leave;
+	    }
+	    merge_keys_and_selfsig( sec_keyblock );
+	    if( fix_keyblock( sec_keyblock ) )
+		sec_modified++;
 	}
-	merge_keys_and_selfsig( sec_keyblock );
-	if( fix_keyblock( sec_keyblock ) )
-	    sec_modified++;
     }
 
     /* and now get the public key */
@@ -636,14 +648,14 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 	modified++;
 
     if( sec_keyblock ) { /* check that they match */
-	/* FIXME: check that they both match */
+	/* fixme: check that they both match */
 	tty_printf(_("Secret key is available.\n"));
     }
 
     toggle = 0;
     cur_keyblock = keyblock;
     for(;;) { /* main loop */
-	int i, arg_number=0;
+	int i, arg_number;
 	char *p;
 
 	tty_printf("\n");
@@ -672,7 +684,7 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 	    trim_spaces(answer);
 	} while( *answer == '#' );
 
-	arg_number = 0;
+	arg_number = 0; /* Yes, here is the init which egcc complains about*/
 	if( !*answer )
 	    cmd = cmdLIST;
 	else if( *answer == CONTROL_D )
@@ -689,10 +701,13 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 		arg_number = atoi(p);
 	    }
 
-	    for(i=0; cmds[i].name; i++ )
+	    for(i=0; cmds[i].name; i++ ) {
 		if( !stricmp( answer, cmds[i].name ) )
 		    break;
-	    if( cmds[i].need_sk && !sec_keyblock ) {
+	    }
+	    if( sign_mode && !cmds[i].signmode )
+		cmd = cmdINVCMD;
+	    else if( cmds[i].need_sk && !sec_keyblock ) {
 		tty_printf(_("Need the secret key to do this.\n"));
 		cmd = cmdNOP;
 	    }
@@ -702,53 +717,15 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 	switch( cmd )  {
 	  case cmdHELP:
 	    for(i=0; cmds[i].name; i++ ) {
-		if( cmds[i].need_sk && !sec_keyblock )
+		if( sign_mode && !cmds[i].signmode )
+		    ;
+		else if( cmds[i].need_sk && !sec_keyblock )
 		    ; /* skip if we do not have the secret key */
 		else if( cmds[i].desc )
 		    tty_printf("%-10s %s\n", cmds[i].name, _(cmds[i].desc) );
 	    }
 	    break;
 
-	  case cmdQUIT:
-	    if( have_commands )
-		goto leave;
-	    if( !modified && !sec_modified )
-		goto leave;
-	    if( !cpr_get_answer_is_yes("keyedit.save.okay",
-					_("Save changes? ")) ) {
-		if( cpr_enabled()
-		    || cpr_get_answer_is_yes("keyedit.cancel.okay",
-					     _("Quit without saving? ")) )
-		    goto leave;
-		break;
-	    }
-	    /* fall thru */
-	  case cmdSAVE:
-	    if( modified || sec_modified  ) {
-		if( modified ) {
-		    rc = update_keyblock( &keyblockpos, keyblock );
-		    if( rc ) {
-			log_error(_("update failed: %s\n"), g10_errstr(rc) );
-			break;
-		    }
-		}
-		if( sec_modified ) {
-		    rc = update_keyblock( &sec_keyblockpos, sec_keyblock );
-		    if( rc ) {
-			log_error(_("update secret failed: %s\n"),
-							    g10_errstr(rc) );
-			break;
-		    }
-		}
-	    }
-	    else
-		tty_printf(_("Key not changed so no update needed.\n"));
-	    rc = update_trust_record( keyblock, 0, NULL );
-	    if( rc )
-		log_error(_("update of trustdb failed: %s\n"),
-			    g10_errstr(rc) );
-	    goto leave;
-
 	  case cmdLIST:
 	    redisplay = 1;
 	    break;
@@ -778,12 +755,14 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 	  case cmdLSIGN: /* sign (only the public key) */
 	    if( count_uids(keyblock) > 1 && !count_selected_uids(keyblock) ) {
 		if( !cpr_get_answer_is_yes("keyedit.sign_all.okay",
-					   _("Really sign all user ids? ")) ) {
-		    tty_printf(_("Hint: Select the user ids to sign\n"));
+					   _("Really sign all user IDs? ")) ) {
+		    tty_printf(_("Hint: Select the user IDs to sign\n"));
 		    break;
 		}
 	    }
-	    sign_uids( keyblock, locusr, &modified, cmd == cmdLSIGN );
+	    if( !sign_uids( keyblock, locusr, &modified, cmd == cmdLSIGN )
+		&& sign_mode )
+		goto do_cmd_save;
 	    break;
 
 	  case cmdDEBUG:
@@ -815,13 +794,13 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 		int n1;
 
 		if( !(n1=count_selected_uids(keyblock)) )
-		    tty_printf(_("You must select at least one user id.\n"));
+		    tty_printf(_("You must select at least one user ID.\n"));
 		else if( count_uids(keyblock) - n1 < 1 )
-		    tty_printf(_("You can't delete the last user id!\n"));
+		    tty_printf(_("You can't delete the last user ID!\n"));
 		else if( cpr_get_answer_is_yes(
 			    "keyedit.remove.uid.okay",
-			n1 > 1? _("Really remove all selected user ids? ")
-			      : _("Really remove this user id? ")
+			n1 > 1? _("Really remove all selected user IDs? ")
+			      : _("Really remove this user ID? ")
 		       ) ) {
 		    menu_deluid( keyblock, sec_keyblock );
 		    redisplay = 1;
@@ -836,7 +815,7 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 		int n1;
 
 		if( !(n1=count_selected_uids(keyblock)) )
-		    tty_printf(_("You must select at least one user id.\n"));
+		    tty_printf(_("You must select at least one user ID.\n"));
 		else if( menu_delsig( keyblock ) ) {
 		    /* no redisplay here, because it may scroll away some
 		     * status output of delsig */
@@ -944,6 +923,53 @@ keyedit_menu( const char *username, STRLIST locusr, STRLIST commands )
 	    }
 	    break;
 
+	  case cmdQUIT:
+	    if( have_commands )
+		goto leave;
+	    if( !modified && !sec_modified )
+		goto leave;
+	    if( !cpr_get_answer_is_yes("keyedit.save.okay",
+					_("Save changes? ")) ) {
+		if( cpr_enabled()
+		    || cpr_get_answer_is_yes("keyedit.cancel.okay",
+					     _("Quit without saving? ")) )
+		    goto leave;
+		break;
+	    }
+	    /* fall thru */
+	  case cmdSAVE:
+	  do_cmd_save:
+	    if( modified || sec_modified  ) {
+		if( modified ) {
+		    rc = update_keyblock( &keyblockpos, keyblock );
+		    if( rc ) {
+			log_error(_("update failed: %s\n"), g10_errstr(rc) );
+			break;
+		    }
+		}
+		if( sec_modified ) {
+		    rc = update_keyblock( &sec_keyblockpos, sec_keyblock );
+		    if( rc ) {
+			log_error(_("update secret failed: %s\n"),
+							    g10_errstr(rc) );
+			break;
+		    }
+		}
+	    }
+	    else
+		tty_printf(_("Key not changed so no update needed.\n"));
+	    /* TODO: we should keep track whether we have changed
+	     *	     something relevant to the trustdb */
+	    if( !modified && sign_mode )
+		rc = 0; /* we can skip at least in this case */
+	    else
+		rc = update_trust_record( keyblock, 0, NULL );
+	    if( rc )
+		log_error(_("update of trustdb failed: %s\n"),
+			    g10_errstr(rc) );
+	    goto leave;
+
+	  case cmdINVCMD:
 	  default:
 	    tty_printf("\n");
 	    tty_printf(_("Invalid command  (try \"help\")\n"));
@@ -1086,7 +1112,7 @@ show_key_with_all_names( KBNODE keyblock, int only_marked,
 		   tty_printf("(%d)* ", i);
 		else
 		   tty_printf("(%d)  ", i);
-		tty_print_string( uid->name, uid->len );
+		tty_print_utf8_string( uid->name, uid->len );
 		tty_printf("\n");
 		if( with_prefs )
 		    show_prefs( keyblock, uid );
@@ -1112,7 +1138,7 @@ show_key_and_fingerprint( KBNODE keyblock )
 	}
 	else if( node->pkt->pkttype == PKT_USER_ID ) {
 	    PKT_user_id *uid = node->pkt->pkt.user_id;
-	    tty_print_string( uid->name, uid->len );
+	    tty_print_utf8_string( uid->name, uid->len );
 	    break;
 	}
     }
@@ -1296,7 +1322,7 @@ menu_delsig( KBNODE pub_keyblock )
 	   int okay, valid, selfsig, inv_sig, no_key, other_err;
 
 	    tty_printf("uid  ");
-	    tty_print_string( uid->name, uid->len );
+	    tty_print_utf8_string( uid->name, uid->len );
 	    tty_printf("\n");
 
 	   okay = inv_sig = no_key = other_err = 0;
@@ -1537,7 +1563,7 @@ menu_select_uid( KBNODE keyblock, int idx )
 	    }
 	}
 	if( !node ) {
-	    tty_printf(_("No user id with index %d\n"), idx );
+	    tty_printf(_("No user ID with index %d\n"), idx );
 	    return 0;
 	}
     }
@@ -1682,8 +1708,8 @@ ask_revoke_sig( KBNODE keyblock, KBNODE node )
     }
 
     tty_printf(_("user ID: \""));
-    tty_print_string( unode->pkt->pkt.user_id->name,
-		      unode->pkt->pkt.user_id->len );
+    tty_print_utf8_string( unode->pkt->pkt.user_id->name,
+			   unode->pkt->pkt.user_id->len );
     tty_printf(_("\"\nsigned with your key %08lX at %s\n"),
 		sig->keyid[1], datestr_from_sig(sig) );
 
@@ -1718,7 +1744,7 @@ menu_revsig( KBNODE keyblock )
 	    PKT_user_id *uid = node->pkt->pkt.user_id;
 	    /* Hmmm: Should we show only UIDs with a signature? */
 	    tty_printf("     ");
-	    tty_print_string( uid->name, uid->len );
+	    tty_print_utf8_string( uid->name, uid->len );
 	    tty_printf("\n");
 	}
 	else if( node->pkt->pkttype == PKT_SIGNATURE
@@ -1755,7 +1781,7 @@ menu_revsig( KBNODE keyblock )
 	if( node->pkt->pkttype == PKT_USER_ID ) {
 	    PKT_user_id *uid = node->pkt->pkt.user_id;
 	    tty_printf("     ");
-	    tty_print_string( uid->name, uid->len );
+	    tty_print_utf8_string( uid->name, uid->len );
 	    tty_printf("\n");
 	}
 	else if( node->pkt->pkttype == PKT_SIGNATURE ) {
