@@ -64,7 +64,7 @@ do_check( PKT_secret_key *sk )
 		keyid_from_sk( sk2, keyid+2 );
 	    free_secret_key( sk2 );
 	}
-	dek = passphrase_to_dek( keyid, sk->protect.algo,
+	dek = passphrase_to_dek( keyid, sk->pubkey_algo, sk->protect.algo,
 				 &sk->protect.s2k, 0 );
 	cipher_hd = cipher_open( sk->protect.algo,
 				 CIPHER_MODE_AUTO_CFB, 1);
@@ -164,7 +164,7 @@ check_secret_key( PKT_secret_key *sk, int n )
     int i;
 
     if( n < 1 )
-	n = 3; /* use the default value */
+	n = opt.batch? 1 : 3; /* use the default value */
 
     for(i=0; i < n && rc == G10ERR_BAD_PASS; i++ ) {
 	if( i )
